*****  Program 2.1  Simulating Pearson Correlation Coefficient Distributions ******;

LIBNAME CORR 'C:\CORR_EG';
%LET NO_SMPL=2000;       			*** macro variable for # of random samples;
                                    *** under each sample size condition;
%MACRO CORR_RDM;
%DO A = 1 %TO 4;                    *** specify four sample size conditions;
  %IF &A=1 %THEN %DO; %LET SMPLSIZE=10;  %END;
  %IF &A=2 %THEN %DO; %LET SMPLSIZE=20;  %END;
  %IF &A=3 %THEN %DO; %LET SMPLSIZE=50;  %END;
  %IF &A=4 %THEN %DO; %LET SMPLSIZE=100; %END;

%DO B=1 %TO &NO_SMPL;   *** # of samples for each sample size conditions;

DATA DAT;               *** generate two random variables (i.e., uncorrelated);
  DO I=1 TO &SMPLSIZE;
    X=RANNOR(0);
	Y=RANNOR(0); 
    OUTPUT;
  END;
                   *** use PROC CORR to get Pearson r, results as SAS dataset PEARSON;
PROC CORR DATA=DAT NOPRINT OUTP=PEARSON;
  VAR X Y;
RUN;
           *** collect Pearson r from each sample, add sample size condition;
		   *** accumulate Pearson r from samples by appending the Pearson r from a 
           *** sample to a SAS system file 'COR_RDM';

DATA PEARSON; SET PEARSON;
  SMPLSIZE=&SMPLSIZE;
  IF _NAME_='X';
  CORR=Y; 
  KEEP CORR SMPLSIZE;
PROC APPEND BASE=CORR.COR_RDM;
%END;
%END;
%MEND CORR_RDM;
%CORR_RDM;
RUN; QUIT;
                     *** obtain descriptive statistics on the Pearson rs;
                     *** under each of the four sample size conditions;
DATA A; SET CORR.COR_RDM;
PROC SORT; BY SMPLSIZE;
PROC MEANS; BY SMPLSIZE;
  VAR CORR;
TITLE1 'DESCRIPTIVE STATS FOR PEARSON RS BETWEEN TWO RANDOM VARIABLES';
TITLE2 'FOR FOUR DIFFERENT SAMPLE SIZE CONDITIONS';
TITLE3 '*************************************************************';
RUN; QUIT;
                                * obtain bar graphs to show the
                                distribution characteristics of
                                Pearson rs under true null hypothesis
                                for each of the four sample size
                                conditions ;

DATA A; SET CORR.COR_RDM;
PROC SORT; BY SMPLSIZE;

AXIS1 LABEL=(HEIGHT=1.0 FONT=TRIPLEX) ORDER=(0 TO 20 BY 5)
      VALUE=(HEIGHT=1.0 FONT=TRIPLEX) MINOR=NONE;
AXIS2 LABEL=(HEIGHT=1.0 FONT=TRIPLEX) VALUE=(HEIGHT=1.0 FONT=TRIPLEX)
      MINOR=NONE;
PATTERN COLOR=BLACK VALUE=X2;

PROC GCHART DATA=A; BY SMPLSIZE;  * use PROC GCHART for nicer graphs;
  VBAR CORR/ TYPE=PERCENT
           MIDPOINTS= -.9 TO .9 BY .05
           RAXIS=AXIS1 MAXIS=AXIS2
           WIDTH=1
           SPACE=1;
RUN; QUIT;
***************************************************************************;
