* Program 3.1: 	Macro EQUIDST;
/**********************************************************/
/* Macro EQUIDST executes an equidistribution test to     */
/* check the uniform distribution of random numbers       */
/* generated by RANUNI.                                   */
/*                                                        */
/* Parameters:                                            */
/* NRANNUM # of random numbers to be generated.           */
/* HNSINT  # highest number of subintervals. The macro    */
/*         calculates the chi-square test for             */
/*         2,3,...,HNSINT divisions of equal subintervals */
/*         of (0,1).                                      */
/* SEED    Seed of function RANUNI.                       */
/**********************************************************/

%MACRO EQUIDST(NRANNUM=,HNSINT=,SEED=0);
  DATA WORK(KEEP=SINTERV X);

       /* generate the requested number of random numbers.*/

       LENGTH SINTERV X 3;
       DO I=1 TO &NRANNUM;
          R=RANUNI(&SEED);

          /* determine the interval number (variable 'X') into */
          /* which the random number falls. Do this for each   */
          /* division (2-subinterval, 3-subinterval,...        */
          /* divisions).                                       */

          DO SINTERV=2 TO &HNSINT;
              X=1+INT(SINTERV*R);
              OUTPUT;
          END;
       END;
       RUN;

  /* determine the frequency of each subinterval by division.  */

  PROC FREQ DATA=WORK;
       TABLE SINTERV*X/LIST OUT=WORK(KEEP=SINTERV COUNT) NOPRINT;
       RUN;

  /* calculate the chi-square test for each division. */

  DATA WORK(KEEP=PVALUE);
       SET WORK;
       BY SINTERV;
       RETAIN EXPFREQ CHISQ;

       /* a division starts. */

       IF FIRST.SINTERV THEN DO; EXPFREQ=&NRANNUM/SINTERV;
                                 CHISQ=0;
                                 END;
       CHISQ=CHISQ+(COUNT-EXPFREQ)**2/EXPFREQ;

       /* the last interval of the division is read. */
       /* determine the p-value of the test.         */

       IF LAST.SINTERV THEN DO; PVALUE=1-PROBCHI(CHISQ,SINTERV-1);
                                OUTPUT;
                                END;
  PROC FORMAT;
     VALUE SIGNIF 0.0000-0.0001='P<0.0001'
                  0.0001-0.001 ='P<0.001'
                  0.001 -0.01  ='P<0.01'
                  0.01  -0.05  ='P<0.05'
                  0.05  -0.10  ='P<0.10'
                  0.10  -0.15  ='P<0.15'
                  0.15  -1     ='P>0.15';

  PROC FREQ DATA=WORK;   *** summarize the results;
     TABLE PVALUE;
     FORMAT PVALUE SIGNIF.;
     TITLE1 "Results of the Uniformity Test";
     TITLE2 "------------------------------";
     TITLE3 "# of Random Numbers: &NRANNUM";
     TITLE4 "Subintervals Tested: 2 to &HNSINT";
  RUN;
%MEND;
*********************************************************;
