
*****  Program 3.12: Generation of 10,000 Random Bond Ratings  ****;

*********************************************************************;
/* Read the theoretical distribution of Moody's */
/* bond ratings as of 12/1998.                  */

DATA BRATINGS;
     INPUT BRATING $3. MDPROB;
     MDPROB=MDPROB/100;
     CARDS;
Aaa     0.707   *** The sum of these frequencies is 100.
Aa      4.302
A      11.903
Baa    17.030
Ba     21.332
B      33.942
Caa    10.785
;
DATA RNDBONDS(KEEP=BRATING);
     SET BRATINGS END=ENDOFRT;

     /* set up arrays for the bond ratings ('BRTNGS') */
     /* and the percent of companies in those rating  */
     /* categories ('PROB').                          */

     ARRAY BRATINGS(7) $5 BRTNGS1-BRTNGS7;
     ARRAY PROB(7) PROB1-PROB7;
     RETAIN BRTNGS1-BRTNGS7 PROB1-PROB7;
     BRATINGS(_N_)=BRATING;
     PROB(_N_)=MDPROB;

     /* generate the 10,000 random bond ratings. */

     IF ENDOFRT THEN DO;
        DO I=1 TO 10000;

           /* function RANTBL is called with the probabilities */
           /* stored in array 'PROB'. The integer returned by  */
           /* RANTBL is mapped into the rating category.       */

           BRATING=BRATINGS(RANTBL(123,OF PROB(*)));
           OUTPUT;
        END;
        END;

/* check the random distribution to the theoretical one using */
/* the chi-square test of PROC FREQ. Obtain the frequencies   */
/* by rating category in the random sample.                   */

PROC FREQ DATA=RNDBONDS;
     TABLE BRATING / NOPRINT OUT=FREQHIST(RENAME=(PERCENT=RNDPNT)
                                 KEEP=BRATING PERCENT COUNT);
     RUN;

/* set up a special file where there is a record for each       */
/* rating category and for each sample (theoretical and random).*/
/* both theoretical and random samples have 10,000 bonds each.  */
/* variable 'FRQ' holds the frequency in each record.           */

PROC SORT DATA=BRATINGS;
     BY BRATING;
DATA BOTH;
     MERGE BRATINGS FREQHIST;
     BY BRATING;
     SAMPLE='Theoretical';
     FRQ=ROUND(10000*MDPROB);
     OUTPUT;
     SAMPLE='Random';
     FRQ=COUNT;
     OUTPUT;
     RUN;

/* execute PROC FREQ to determine whether or not the   */
/* random sample is from the theoretical distribution. */

PROC FREQ DATA=BOTH;
     TABLE SAMPLE*BRATING / CHISQ;
     WEIGHT FRQ;
     RUN;
***********************************************************************;
