
*****  Program 3.13: Generating Random Stock Prices with RANTBL  ****;
/* determine the probability mass function of the stock prices. */
/* file 'SPPRICE' was created in Program 3.10.                  */

PROC FREQ DATA=SPPRICE;
     TABLE PRICE / NOPRINT OUT=SPFREQ(KEEP=PRICE PERCENT);

/* macro variable 'N' holds the number of unique stock prices.  */

DATA _NULL_;
     IF 0 THEN SET SPFREQ NOBS=N;
     IF _N_=1 THEN CALL SYMPUT('N',COMPRESS(PUT(N,8.0)));
     STOP; RUN;

/* generate 1,000 random stock prices according to the       */
/* probabilities determined before (file 'RNDPRICE'). File   */
/* 'SPCDF' holds the theoretical cumulative density function */
/* of the S&P 500 stock prices.                              */

DATA RNDPRICE(KEEP=PRICE) SPCDF(KEEP=PRICE SPPROB);
     SET SPFREQ END=ENDOFPRC;

     /* arrays 'PRICES' and 'PROB' hold the unique stock prices */
     /* and their probabilities of the S&P 500 stocks. Load     */
     /* them into the arrays from the output of 'PROC FREQ'.    */

     ARRAY PRICES(&N) PRC1-PRC&N;
     ARRAY PROB (&N) PRB1-PRB&N;
     RETAIN PRC1-PRC&N PRB1-PRB&N;
     PRICES(_N_)=PRICE;
     PROB(_N_)=PERCENT/100;

     /* the arrays are loaded. Create the theoretical cdf (file */
     /* 'SPCDF') and generate the 1,000 random stock prices.    */

     IF ENDOFPRC THEN DO;

        /* calculate and save the theoretical cdf. */

        SPPROB=0;
        DO I=1 TO DIM(PRICES);
           PRICE=PRICES(I);
           SPPROB=SPPROB+PROB(I);
           OUTPUT SPCDF;
        END;

        /* generate random stock prices using function RANTBL. */

        DO I=1 TO 1000;

           /* function RANTBL is called with the probabilities */
           /* in array 'PROB'. The returned integer is mapped  */
           /* to the actual stock price.                       */

           PRICE=PRICES(RANTBL(123,OF PROB(*)));
           OUTPUT RNDPRICE;
        END;
     END;
     RUN;

/* determine the cumulative distribution */
/* function of the random stock prices.  */

PROC FREQ DATA=RNDPRICE;
     TABLE PRICE / NOPRINT OUT=RNDCDF(KEEP=PRICE PERCENT);
     RUN;
DATA RNDCDF(KEEP=PRICE RNDPROB);
     SET RNDCDF;
     RETAIN RNDPROB 0;
     RNDPROB=RNDPROB+PERCENT/100;
     RUN;

/* concatenate the theoretical (file 'SPCDF') and the randomly  */
/* generated (file 'RNDCDF') cdf's. Set up a sample identifier. */

DATA BOTH;
     SET SPCDF(RENAME=(SPPROB=PROB))
         RNDCDF(RENAME=(RNDPROB=PROB) IN=RND);
     RETAIN SAMPLE 'Theoretical';
     IF RND THEN SAMPLE='Random';
     OUTPUT;
     RUN;

/* create an annotate file for the legend. */

DATA ANNO;
     XSYS='2'; YSYS='2';
     FUNCTION='LABEL'; POSITION='6'; STYLE='DUPLEX'; SIZE=1.2;
     X=170; Y=0.20; TEXT='THEORETICAL'; OUTPUT;
            Y=0.10; TEXT='RANDOM';      OUTPUT;
     LINE=1;  FUNCTION='MOVE'; X=150; Y=0.1; OUTPUT;
              FUNCTION='DRAW'; X=168; Y=0.1; OUTPUT;
     LINE=20; FUNCTION='MOVE'; X=150; Y=0.2; OUTPUT;
              FUNCTION='DRAW'; X=168; Y=0.2; OUTPUT;

/* plot the two cdf's. */

PROC GPLOT DATA=BOTH ANNOTATE=ANNO;
     PLOT PROB*PRICE=SAMPLE / VAXIS=AXIS1 HAXIS=AXIS2 NOLEGEND;
     AXIS1 LABEL=(A=90 R=0 F=DUPLEX H=1.2 'Probability')
           ORDER=0 TO 1 BY 0.2 VALUE=(H=1.2 F=DUPLEX) MINOR=(N=1);
     AXIS2 LABEL=(F=DUPLEX H=1.2 'Stock Price ($)')
           ORDER=0 TO 200 BY 25 VALUE=(H=1.2 F=DUPLEX) MINOR=(N=3);
     TITLE  F=SWISS H=1.3 'Theoretical and Randomly Generated Cumulative Distribution Functions';
     TITLE2 F=SWISS H=1.3 'Size of Random Sample is 1,000. Function Used: RANTBL';
     SYMBOL1 I=JOIN C=BLACK R=1 L=1;
     SYMBOL2 I=JOIN C=BLACK R=1 L=20;
     RUN;
     QUIT;
*************************************************************************************;
