
******   Program 6.1: SAS Program for T-Test - First Approach   *********;
LIBNAME TTEST 'C:\T_TEST\TRIALS';

            * -- to avoid the problem of SAS Log Window becomes full;
PROC PRINTTO LOG='C:\T_TEST\TRIALS\LOGFILE.TMP';
RUN;

%MACRO TTEST;     * beginning of the macro program 'TTEST';

%DO A=1 %TO 2;    * A=1:equal variance, A=2:unequal;
%DO B=1 %TO 2;    * B=1:equal sample size, B=2:unequal;

%DO I=1 %TO 10000;   * number of replications in each cell. 10,000 in this case;

DATA TTEST;

        /***  set parameters  ***/
ALPHA=0.05;    * nominal TYPE I error rate;
MEAN=50;       * common mean for two populations;
SD1=10;        * STD for GRP 1 & 2 - equal variance condition;
N1=20;         * sample size for GRP 1 & 2 - equal N condition;
        /***  end of parameters ***/

  IF &A=1 THEN SD2=SD1;      * A=1:equal variance, A=2:unequal - 1.5 times sd1;
     ELSE SD2=SD1*1.5;

  IF &B=1 THEN N2=N1;        * B=1:equal sample size, B=2:unequal - 2 times n1;
     ELSE N2=N1*2;
                             * initiate the accumulators;
 SUM1=0;      * group 1:  sum of x - sigma(x);
 SUM2=0;      * group 2:  sum of x - sigma(x);
 SSU1=0;      * group 1:  uncorrected sum of squares (ss);
 SSU2=0;      * group 2:  uncorrected sum of squares (ss);
                     
   DO I=1 TO N1;          *  generate Group 1 data, carry out some computations;
      X1=MEAN + SD1*RANNOR(0);
      SUM1=SUM1 + X1;
      SSU1=SSU1 + X1**2;
   END;
                     
   DO J=1 TO N2;          *  generate Group 2 data, carry out some computations;
      X2=MEAN + SD2*RANNOR(0);
      SUM2=SUM2 + X2;
      SSU2=SSU2 + X2**2;
   END;
                          
   SSC1 = SSU1 - ((SUM1**2)/N1);     * compute corrected SS for two groups;
   SSC2 = SSU2 - ((SUM2**2)/N2);

   MEAN1=SUM1/N1;  MEAN2=SUM2/N2;    * means of the two samples;
                          
   VARP=(SSC1+SSC2)/(N1+N2-2);       * pooled variance;
                          
   T=(MEAN1-MEAN2)/SQRT(VARP*(1/N1 + 1/N2));    * t statistic;

   DF = N1 + N2 - 2;                  * degrees of freedom;
                         
   IF T<0 THEN PT=2*PROBT(T,DF);         * p value of the t statistic - two-tailed;
   ELSE IF T>0 THEN PT=2*(1-PROBT(T,DF));
                           
   IF PT<ALPHA THEN SIG=1;      * classify each t-test as either significant or not;
      ELSE SIG=0;
                         
   IF &A=1 THEN EQ_VAR='  EQUAL';   * add design conditions to the output data set;
      ELSE EQ_VAR='UNEQUAL';

   IF &B=1 THEN EQ_N='  EQUAL';
      ELSE EQ_N='UNEQUAL';
DATA NEW; SET TTEST;                * append the results to a SAS data set on disk;
  KEEP T DF PT SIG EQ_VAR EQ_N;
  PROC APPEND BASE=TTEST.TTEST;
RUN;

%END;                * end iteration do loop;
%END;                * end B do loop;
%END;                * end A do loop;
%MEND TTEST;         * close the macro program;
%TTEST;              * run the macro program;
RUN;
                     * check empirical rejection rate for each cell condition;
DATA A; SET TTEST.TTEST;  
PROC SORT; BY EQ_VAR EQ_N;
PROC FREQ; BY EQ_VAR EQ_N;
  TABLES SIG;
RUN;
******************************************************************************;
