
******   Program 6.2: SAS Program for T-Test - Second Approach   *********;
LIBNAME TTEST 'C:\T_TEST\TRIALS';

            * -- to avoid the problem of SAS Log Window becomes full;
PROC PRINTTO LOG='C:\T_TEST\TRIALS\LOGFILE.TMP';
RUN;

%MACRO TTEST;

%DO A=1 %TO 2;    * A=1:equal variance, A=2:unequal;
%DO B=1 %TO 2;    * B=1:equal sample size, B=2:unequal;

%DO REP=1 %TO 10000;  * number of replications in each cell;

PROC IML;

/*** define parameters  ***/

%LET ALPHA=0.05;    * nominal Type I error rate;
MEAN=50;            * common mean for two populations;
SD1=10;             * STD for GRP 1 & 2 - equal variance condition;
N1=20;              * sample size for GRP 1 & 2 - equal N condition;

/*** end of parameters ***/

                    * A=1:equal variance, A=2:unequal - 1.5 times SD1;
  IF &A=1 THEN SD2=SD1;
     ELSE IF &A=2 THEN SD2=SD1*1.5;

                    * B=1:equal sample size, B=2:unequal - 2 times N1;
  IF &B=1 THEN N2=N1;
     ELSE IF &B=2 THEN N2=N1*2;

                      * generate group 1 data;
DAT1=SD1*RANNOR(J(N1,1,0)) + MEAN;
GRP1=J(N1,1,1);                    * assigning group number: group=1;
DAT1=DAT1||GRP1;                   * horizontal concatenation;

                      * generate group 2 data;
DAT2=SD2*RANNOR(J(N2,1,0)) + MEAN;
GRP2=J(N2,1,2);                    * assigning group number: group=2;
DAT2=DAT2||GRP2;

DATA=DAT1//DAT2;       * vertical concatenation - put data of both groups together;

CREATE DATAALL FROM DATA[COLNAME={X GROUP}];   * create a SAS working data set;
APPEND FROM DATA;
                                    * direct the output to a file on disk;
FILENAME NEWOUT 'C:\T_TEST\TRIALS\OUTFILE';
PROC PRINTTO PRINT=NEWOUT NEW;
RUN;
                    * run PROC T-TEST procedure;
PROC TTEST DATA=DATAALL;
  CLASS GROUP;
  VAR X;
RUN;
                    * redirect output to print;
PROC PRINTTO PRINT=PRINT; RUN;

                    * read proc ttest output from the disk file and
                      obtain t statistic, df, and p value;
DATA READIN; INFILE NEWOUT;
   INPUT WORD $ @@;
IF WORD='Equal' THEN DO;
  INPUT T DF PT;
                    * add simulation design information to data;
   IF PT<&ALPHA THEN SIG=1;
      ELSE SIG=0;
   IF &A=1 THEN EQ_VAR='  EQUAL';
      ELSE EQ_VAR='UNEQUAL';
    IF &B=1 THEN EQ_N='  EQUAL';
      ELSE EQ_N='UNEQUAL';
  OUTPUT;
  KEEP T DF PT SIG EQ_VAR EQ_N;
END;
RUN;

                 * append the relevant information to a SAS system file;
DATA NEW; SET READIN;
  KEEP T DF PT SIG EQ_VAR EQ_N;
  PROC APPEND BASE=TTEST.TTEST;
RUN;

%END;              * end replication do loop;
%END;              * end B do loop;
%END;              * end A do loop;
%MEND TTEST;       * close the macro program 'TTEST';
%TTEST;            * Run macro program 'TTEST';
RUN;

PROC SORT; BY EQ_VAR EQ_N;
PROC FREQ; BY EQ_VAR EQ_N;    * check Type I error rate in four cells;
  TABLES SIG;
RUN;
*************************************************************************;
