
*******  Program 6.3 SAS Program for ANOVA Example  *************;
LIBNAME ANOVA 'C:\ANOVA\TRIALS';

    * -- to avoid the problem of SAS Log Window becomes full;
PROC PRINTTO LOG='C:\ANOVA\TRIALS\LOGFILE.TMP';
RUN;

%MACRO ANOVA;

%DO A=1 %TO 2;        * A=1: normal data,  A=2: non-normal data;
%DO B=1 %TO 2;        * B=1: equal variance,  B=2: unequal variance;
%DO REP=1 %TO 5000;   * 5,000 replications in each cell;

%LET ALPHA=0.05;   * nominal Type I error rate;

PROC IML;

MEAN=50;           * common mean for 3 groups;
N=30;              * common sample size for 3 groups;

                   * Fleishman coefficients for data shapes
                     1st row: normal data, 2nd row: non-normal data;
DIST={1 0 0,
      .92966052480111 .39949667453766 -.03646699281275};

                   * variances of 3 groups
                     1st row: equal variances, 2nd row: unequal;
VAR={10 10 10,
     10 20 40};
                   * generate data for group 1;
X=RANNOR(J(N,1,0));
X=-DIST[&A,2] + DIST[&A,1]*X + DIST[&A,2]*X##2 + DIST[&A,3]*X##3;
X=X*SQRT(VAR[&B,1]) + MEAN;
GRP=J(N,1,1);               * assign group number: group=1;
GROUP1=X||GRP;

                   * generate data for group 2;
X=RANNOR(J(N,1,0));
X=-DIST[&A,2] + DIST[&A,1]*X + DIST[&A,2]*X##2 + DIST[&A,3]*X##3;
X=X*SQRT(VAR[&B,2]) + MEAN;
GRP=J(N,1,2);               * assign group number: group=2;
GROUP2=X||GRP;

                   * generate data for group 3;
X=RANNOR(J(N,1,0));
X=-DIST[&A,2] + DIST[&A,1]*X + DIST[&A,2]*X##2 + DIST[&A,3]*X##3;
X=X*SQRT(VAR[&B,3]) + MEAN;
GRP=J(N,1,3);               * assign group number: group=3;
GROUP3=X||GRP;

                  * combine 3 groups data, vertical concatenation;
DATA=GROUP1//GROUP2//GROUP3;

                             * create SAS working data;
CREATE DATAALL FROM DATA[COLNAME={X GROUP}];
APPEND FROM DATA;

                   * run ANOVA analysis, and output ANOVA results
                     to a SAS working data 'ANOVAOUT';

PROC ANOVA DATA=DATAALL NOPRINT OUTSTAT=ANOVAOUT;
  CLASS GROUP;
  MODEL X=GROUP;
RUN;
                    * use 'ANOVAOUT' data;
                    * extract relevant ANOVA results;

DATA AA; SET ANOVAOUT;
  IF _TYPE_='ANOVA';
  DF_MOD=DF; SS_MOD=SS;

                    * add a variable indicating statistical significance;
  IF PROB<&ALPHA THEN SIG='YES';
     ELSE SIG=' NO';

  KEEP DF_MOD SS_MOD F PROB SIG;     * keep relevant variables;

                    * extract error df, error sum-of-squares;

DATA BB; SET ANOVAOUT;
  IF _TYPE_='ERROR';
  DF_ERR=DF; SS_ERR=SS;
  KEEP DF_ERR SS_ERR;

                   * merge two data sets, add study design information;
DATA AB; MERGE AA BB;
  IF &A=1 THEN NORMAL='YES';
     ELSE IF &A=2 THEN NORMAL=' NO';
  IF &B=1 THEN EQ_VAR='YES';
     ELSE IF &B=2 THEN EQ_VAR=' NO';
                    * append each replication results to a permanent SAS data;
PROC APPEND BASE=ANOVA.ANOVA;

%END;         * close replication do loop;
%END;         * close B do loop;
%END;         * close A do loop;
%MEND ANOVA;  * macro 'ANOVA';
%ANOVA;       * run macro 'ANOVA';
RUN;

              * obtain descriptive statistics for the simulation results;
DATA A; SET ANOVA.ANOVA;
PROC SORT; BY NORMAL EQ_VAR;
PROC FREQ; BY NORMAL EQ_VAR;
  TABLES SIG;
RUN;
******************************************************************;
