
****  Program 6.4 Simulating Correction Formulas for Regression Sample R2 Bias  ****;
LIBNAME REG 'C:\REG\TRIALS';

    * -- to avoid the problem of SAS Log Window becomes full;
PROC PRINTTO LOG='C:\REG\TRIALS\LOGFILE.TMP';
RUN;
                    * population correlation matrix, with population R_square=0.50;
                    * multicollinearity r=0.30, 8 predictors;
DATA A (TYPE=CORR);
_TYPE_='CORR';
INPUT X1 X2 X3 X4 X5 X6 X7 X8 Y;
CARDS;
1.00 . . . . . . . .
0.30 1.00 . . . . . . .
0.30 0.30 1.00 . . . . . .
0.30 0.30 0.30 1.00 . . . . .
0.30 0.30 0.30 0.30 1.00 . . . .
0.30 0.30 0.30 0.30 0.30 1.00 . . .
0.30 0.30 0.30 0.30 0.30 0.30 1.00 . .
0.30 0.30 0.30 0.30 0.30 0.30 0.30 1.00 .
.44019 .44019 .44019 .44019 .44019 .44019 .44019 .44019 1.00
;
       * obtain factor pattern matrix for later data generation;
PROC FACTOR N=9 OUTSTAT=FACOUT;     
DATA PATTERN; SET FACOUT;
  IF _TYPE_='PATTERN';
  DROP _TYPE_ _NAME_;
RUN;

%MACRO REG;          * starts the macro program 'reg';

                     * 3 sample size conditions: A=1:N=20, A=2:N=40, A=3:N=80;
%DO A=1 %TO 3;
    %IF &A=1 %THEN %DO; %LET N=20; %END;
    %IF &A=2 %THEN %DO; %LET N=40; %END;
    %IF &A=3 %THEN %DO; %LET N=80; %END;

%DO REP=1 %TO 2000;               * number of replications in each cell;

                                  * generate correlated sample data of 9 variables;
PROC IML;                          
   USE PATTERN;                   * read in the factor pattern as a matrix 'F';
   READ ALL VAR _NUM_ INTO F;
  F=F`;                           * transpose 'F' for later premultiplication;

DAT=RANNOR(J(&N,9,0));            * generate 9 random variables (&Nx9 dimension);
DAT=DAT`;                         * transpose the random data matrix (9x&N dimension);
DAT=F*DAT;                        * premultiply 'F' to 'DAT', variables become correlated;
DAT=DAT`;                         * transpose the data matrix back (&Nx9 dimension);

                                  * create a sas working data set 'REGDATA';

CREATE REGDATA FROM DAT[COLNAME={X1 X2 X3 X4 X5 X6 X7 X8 Y}];
APPEND FROM DAT;

                                    * use SAS PROC REG to obtain sample r-square;
                                    * output the results to SAS working data 'REGOUT';

PROC REG DATA=REGDATA NOPRINT OUTEST=REGOUT;
  MODEL Y =X1 X2 X3 X4 X5 X6 X7 X8 / SELECTION=RSQUARE;
RUN;
                                    * use 'REGOUT' data;
DATA A; SET REGOUT;
  IF _IN_=8;                        * select the row of data contains sample r-square;
  P=8; N=&N; NP_RATIO=N/P;          * add study design features: N, P N/P RATIO;
  RSQ=_RSQ_;
  RSQ_ADJ1=1-(N/(N-P))*(1-RSQ);     * apply four correction formulas;
  RSQ_ADJ2=1-((N-1)/(N-P-1))*(1-RSQ);
  RSQ_ADJ3=1-((N-1)/(N-P))*(1-RSQ);
  RSQ_ADJ4=RSQ-((P-2)/(N-P-1))*(1-RSQ)-(2*(N-3))/((N-P-1)*(N-P+1))*(1-RSQ)**2;

 * obtain bias: deviation of r-square and adjusted r-squares from population r-square;

  BIAS_RSQ=RSQ-0.50;               
  BIAS1=RSQ_ADJ1-0.50;
  BIAS2=RSQ_ADJ2-0.50;
  BIAS3=RSQ_ADJ3-0.50;
  BIAS4=RSQ_ADJ4-0.50;
                           * only keep the relevant information for the study;
KEEP N P NP_RATIO RSQ RSQ_ADJ1 RSQ_ADJ2 RSQ_ADJ3 RSQ_ADJ4 BIAS_RSQ BIAS1 BIAS2 BIAS3 BIAS4;

                     * append the results from each replication to permanent SAS data; 

PROC APPEND BASE=REG.REG8_RSQ;

%END;             * close &A do loop;
%END;             * close &REP do loop;
%MEND REG;        * end the macro program 'REG';
%REG;             * run the macro 'REG' program;
RUN;

DATA A; SET REG.REG8_RSQ;
PROC SORT; BY NP_RATIO;         * analyze the results, obtain the average bias;
PROC MEANS; BY NP_RATIO;
  VAR BIAS_RSQ BIAS1 BIAS2 BIAS3 BIAS4;
RUN;
************************************************************************************;
