/************************************************************************************/
/* Macro AR1 generates a lag one autoregressive process 200 times for each of six	*/
/* research conditions.  In a true Monte Carlo study more than six conditions	*/
/* ordinarily will be planned.  For each additional AR1 process desired, add	*/
/* another AR1 statement with the parameters of interest. If so many AR1 statements	*/
/* are added that the program lacks sufficient memory to run, the research plan may	*/
/* be accomplished with /more than one run, each run responsible for some set of	*/
/* conditions.	*/
/*	*/
/* Parameters:
/* N	The number of observations in the series.	*/
/* ARLAG	The squared value of the lag relationship desired.	*/
/* VARIANCE	The squared value of the error variance desired.	*/
/* SCENARI	The designated number id of a given condition.	*/
/*	*/
/************************************************************************************/

OPTIONS LINESIZE=100 NOSOURCE NOSOURCE2 NONOTES; *** Log file will report errors only;

LIBNAME AUTOREG 'C:\MY DOCUMENTS\MY SAS FILES\RESULTS';

PROC FORMAT;VALUE SCENE	*** Labels six conditions;
             1 = 'ARLAG**2 = .40, N = 100'
             2 = 'ARLAG**2 = .60, N = 100'
             3 = 'ARLAG**2 = .80, N = 100'
             4 = 'ARLAG**2 = .40, N = 500'
             5 = 'ARLAG**2 = .60, N = 500'
             6 = 'ARLAG**2 = .80, N = 500';

%MACRO AR1(N,ARLAG,VARIANCE,SCENARI);	*** Macro begins;
  %DO J=1 %TO 200;	*** 200 replications;

    DATA SEM&J;
      ARRAY SERIE SERIE1-SERIE&N;

  SERIE(1)=RANNOR(-1);	*** Generates AR1 data;
    DO J=2 TO &N;
        SERIE(J)= SQRT(&ARLAG)*SERIE(J-1) + SQRT(&VARIANCE)*RANNOR(-1);

END;OUTPUT;
	*** Transposes series to a column;
       KEEP SERIE1-SERIE&N; PROC TRANSPOSE OUT=D2;

DATA D2;SET D2;

PROC ARIMA DATA=D2;IDENTIFY VAR=COL1 NLAG=1 NOPRINT; *** Output is suppressed;
                   ESTIMATE P=1 OUTSTAT=ARIMA1 OUTMODEL=ARIMA2 NOPRINT MAXIT=5000;
	*** Select statistics are outputted;
DATA ARIMA1;SET ARIMA1;KEEP _VALUE_ _STAT_;IF 1<=_N_<= 2;
            PROC TRANSPOSE OUT=ARIMA1;
DATA ARIMA1;SET ARIMA1; AIC=COL1; SBC=COL2;OUTPUT;
            KEEP AIC SBC;
DATA ARIMA2;SET ARIMA2;IF _N_= 6;ARLAG=_VALUE_;ARLAGSTD=_STD_;
            TTEST=ARLAG/ARLAGSTD;OUTPUT;KEEP ARLAG TTEST;
DATA COMBINE;MERGE ARIMA1 ARIMA2;SCENARIO=&SCENARI;OUTPUT; 
	*** OUTMODEL and OUTSTAT results are merged;

   PROC APPEND BASE=AUTOREG.RESULT1 (CNTLLEV=MEMBER); *** Results compiled in library;

 %END;
%MEND COMPUTE; 	*** End of Macro;

%AR1 (100,.40,.60,1);	*** Each AR1 presents one research situation;
%AR1 (100,.60,.40,2);
%AR1 (100,.80,.20,3);
%AR1 (500,.40,.60,4);
%AR1 (500,.60,.40,5);
%AR1 (500,.80,.20,6);

FORMAT SCENARIO SCENE.;

PROC SORT;BY SCENARIO;
PROC SUMMARY PRINT VARDEF=N MAXDEC=2 FW=8;CLASS SCENARIO;  *** Summarizes Results;
             VAR AIC SBC ARLAG TTEST;
RUN;
*********************************************************************************;
