/************************************************************************************/
/* This macro generates 200 replications of data comprised of two predictors and    */
/* one criterion variable and aggregates the results.  The squared Beta weight for  */
/* predictor A is.35; for predictor B, .25.  PROC UNIVARIATE is used to display the */
/* sampling distribution of each statistic.  Match the squared Beta weights used to */
/* generate the data with the estimated Beta weights.                               */
/*                                                                                  */
/************************************************************************************/

OPTIONS LINESIZE=100 NONUMBER NODATE SPOOL ERASE NOSOURCE NOSOURCE2 NONOTES;

LIBNAME REG 'C:\MY DOCUMENTS\MY SAS FILES\RESULTS';

%MACRO REG (REPS,N,ERR,BETA1,BETA2);
   %DO J=1 %TO &REPS;

DATA GENERATE&J;

    DO I=1 TO &N;

       A=RANNOR(-2);         *** predictor a generated to have unit variance;
       B=RANNOR(-3);         *** predictor b generated to have unit variance;
                             *** weighted predictors a and b plus random error give y;
       Y=RANNOR(-6)*SQRT(&ERR) + A*SQRT(&BETA1) + B*SQRT(&BETA2);
       OUTPUT;
    END;

DATA GENERATE&J;SET GENERATE&J;

PROC REG NOPRINT OUTEST=GENERATE&J;   *** proc reg outputs the beta coefficients;
  MODEL Y = A B/OUTSTB;
 RUN;

DATA GENERATE&J;SET GENERATE&J;KEEP A B A_SQ B_SQ;
A_SQ=A**2; B_SQ=B**2;OUTPUT;

DATA GENERATE&J;SET GENERATE&J;KEEP A_SQ B_SQ;

   PROC APPEND BASE=REG.RESULT1 (CNTLLEV=MEMBER);
   PROC DELETE DATA=GENERATE&J;

  %END;
%MEND REG;

   *** 200 reps, n= 500, .45= squared error, .35= beta a squared, .25= beta b squared;

%REG (200,500,.45,.30,.25); 

DATA REG;SET REG.RESULT1;

PROC UNIVARIATE PLOT; VAR A_SQ B_SQ;  *** proc univariate displays the distributions;

QUIT;
*************************************************************************************;

