/************************************************************************************/
/* THIS MACRO GENERATES 200 REPLICATIONS OF DATA COMPRISED OF TWO PREDICTORS AND    */
/* ONE CRITERION VARIABLE.  THE CRITERION VARIABLE HAS A LAG 1 AUTOREGRESSIVE       */
/* PROCESS RUNNING THROUGH IT.  THIS EXAMPLE DIRECTLY BUILDS UPON THE REGRESSION    */
/* PROGRAM PRESENTED IN PROGRAM 9.5.  THE SQUARED BETA WEIGHT FOR PREDICTOR A IS.35;*/
/* FOR PREDICTOR B, .25. THE AR1 COEFFICIENT IS EITHER .80 OR .20, DEPENDING UPON   */
/* WHICH OF THE SIX CONDITIONS ARE EXAMINED.                                        */
/*                                                                                  */
/************************************************************************************/

OPTIONS LINESIZE=100 NOSOURCE NOSOURCE2 NONOTES;

LIBNAME AUTOREG 'C:\MY DOCUMENTS\MY SAS FILES\RESULTS';

%MACRO AUTOREG (REPS,N,RES,AR,ERR,BETA1,BETA2,SCENARI);
   %DO J=1 %TO &REPS;

DATA GENERATE&J;

  ARRAY SERIEA SERIEA1-SERIEA&N;

  SERIEA(1)=RANNOR(-1);

    DO J=2 TO &N;

       SERIEA(J)=RANNOR(-11)*SQRT(&RES) + SERIEA(J-1)*SQRT(&AR);  *** ar1 process;

    END; KEEP SERIEA1-SERIEA&N;OUTPUT;

DATA GENERATE&J; SET GENERATE&J;
   PROC TRANSPOSE OUT=GENERATE&J;   *** move the data from horizontal to vertical;

DATA GENERATE&J;SET GENERATE&J;ID=_N_;SERIESA=COL1;OUTPUT;DROP COL1 _NAME_;

DATA GENERATE;

    DO ID=1 TO &N;

       A=RANNOR(-2);          *** predictor a generated to have unit variance;
       B=RANNOR(-3);          *** predictor b generated to have unit variance;

                                                                *** weighted predictors a and b ? no error yet;
       Y=A*SQRT(&BETA1) + B*SQRT(&BETA2);

OUTPUT;
    END;

DATA GENERATE&J;MERGE GENERATE GENERATE&J;BY ID;

Y = Y + SERIESA*SQRT(&ERR);OUTPUT;          *** add the ar1 process to the criterion;

DATA GENERATE&J;SET GENERATE&J;

PROC AUTOREG NOPRINT DATA=GENERATE&J OUTEST=GENERATE&J;  *** produces estimates;
MODEL Y = A B / ALL NLAG=1 LAGDEP DW=1 DWPROB ;

DATA GENERATE&J;SET GENERATE&J;KEEP A B A_SQ B_SQ _A_1 AR1_SQ;
A_SQ=A**2; B_SQ=B**2; AR1_SQ=_A_1**2;_A_1=ABS(_A_1); OUTPUT; *** obtain square values;

DATA GENERATE&J;SET GENERATE&J;SCENARIO=&SCENARI;OUTPUT;
     KEEP A_SQ B_SQ AR1_SQ _A_1 A B SCENARIO;

   PROC APPEND BASE=AUTOREG.RESULT1 (CNTLLEV=MEMBER);      *** accumulate the results;
   PROC DELETE DATA=GENERATE&J;

  %END;
%MEND AUTOREG;

%AUTOREG (200, 50,.80,.20,.45,.30,.25,1); *** parameters for six different conditions;
%AUTOREG (200,100,.80,.20,.45,.30,.25,2);
%AUTOREG (200,500,.80,.20,.45,.30,.25,3);
%AUTOREG (200, 50,.20,.80,.45,.30,.25,4);
%AUTOREG (200,100,.20,.80,.45,.30,.25,5);
%AUTOREG (200,500,.20,.80,.45,.30,.25,6);

DATA AUTOREG;SET AUTOREG.RESULT1;         *** accessing the output file;

PROC SUMMARY PRINT VARDEF=N MAXDEC=2 FW=8;   *** summarize results for six conditions;
        CLASS SCENARIO;  *** summarizes results;
             VAR  A_SQ B_SQ AR1_SQ _A_1 A B;

             TITLE1  '*****  SCENARIO 1 (N= 50)  *****';
             TITLE2  '*****  SCENARIO 2 (N=100)  *****';
             TITLE3  '*****  SCENARIO 3 (N=500)  *****';

QUIT;
*******************************************************************************;
