/**  PROGRAM 9.7:  Monte Carlo Example: Autocorrelation Affects Regression Results */
/*                                                                                  */
/* This macro generates 200 replications of data comprised of two predictors and    */
/* one criterion variable.  The criterion variable has a lag 1 autoregressive       */
/* process running through it.  This example directly builds upon the regression    */
/* program presented in Program 9.6.  The squared Beta weight for predictor A is.35;*/
/* for predictor B, .25. The AR1 coefficient ranges from .00 to .75, depending upon */
/* which of the sixteen conditions are examined.                                    */
/*                                                                                  */
/************************************************************************************/

OPTIONS LINESIZE=100 NOSOURCE NOSOURCE2 NONOTES;

LIBNAME AUTOREG 'C:\MY DOCUMENTS\MY SAS FILES\RESULTS';

%MACRO AUTOREG (REPS,N,RES,AR,ERR,BETA1,BETA2,SCENARI);
   %DO J=1 %TO &REPS;

DATA GENERATE&J;

  ARRAY SERIEA SERIEA1-SERIEA&N;

  SERIEA(1)=RANNOR(-1);

    DO J=2 TO &N;

       SERIEA(J)=RANNOR(-11)*SQRT(&RES) + SERIEA(J-1)*SQRT(&AR);  *** ar1 process;

    END; KEEP SERIEA1-SERIEA&N; OUTPUT;

DATA GENERATE&J; SET GENERATE&J;
   PROC TRANSPOSE OUT=GENERATE&J;   *** move the data from horizontal to vertical;

DATA GENERATE&J; SET GENERATE&J; ID=_N_; SERIESA=COL1;OUTPUT; DROP COL1 _NAME_;

DATA GENERATE;

    DO ID=1 TO &N;

       A=RANNOR(-2);           *** predictor a generated to have unit variance;
       B=RANNOR(-3);           *** predictor b generated to have unit variance;

                               *** weighted predictors a and b - no error yet;
       Y=A*SQRT(&BETA1) + B*SQRT(&BETA2);

OUTPUT;
    END;

DATA GENERATE&J; MERGE GENERATE GENERATE&J; BY ID;

AAR=A; BAR=B;
Y = Y + SERIESA*SQRT(&ERR); OUTPUT;        *** add the ar1 process to the criterion;

DATA GENERATE&J; SET GENERATE&J;


   *** proc reg outputs the beta coefficients;

PROC REG DATA=GENERATE&J NOPRINT OUTEST=GENERATE ;
  MODEL Y = A B/OUTSTB OUTSEB;
 RUN;

   *** produces estimates;

PROC AUTOREG DATA=GENERATE&J NOPRINT OUTEST=GENERATE&J COVOUT;
  MODEL Y = AAR BAR / ALL NLAG=1 LAGDEP DW=1 DWPROB;

DATA GENERATEA; SET GENERATE;IF _N_=1; KEEP _RMSE_ A B MSE;
    MSE = _RMSE_**2; OUTPUT;

DATA GENERATEB; SET GENERATE; IF _N_=2; KEEP A B STERR_A STERR_B;
    STERR_A = A; STERR_B = B; OUTPUT;

DATA GENERATEB; SET GENERATEB; DROP A B;
DATA GENERATEA1; SET GENERATE&J; IF _N_=1; KEEP AAR BAR _A_1 _MSE_ AR_MSE;
     AR_MSE = _MSE_;OUTPUT;
DATA GENERATEB1; SET GENERATE&J; IF _N_=3; KEEP _STDERR_ STERRAAR;
     STERRAAR = _STDERR_; OUTPUT;
DATA GENERATEC1; SET GENERATE&J; IF _N_=4; KEEP _STDERR_ STERRBAR;
     STERRBAR = _STDERR_;OUTPUT;

DATA GENERATE&J;
MERGE GENERATEA1 GENERATEB1 GENERATEC1 GENERATEA GENERATEB;
KEEP A B A_SQ B_SQ AAR BAR AAR_SQ BAR_SQ _A_1 AR1_SQ
     MSE STERR_A STERR_B AR_MSE STERRAAR STERRBAR;

A_SQ=A**2; B_SQ=B**2; AAR_SQ=AAR**2; BAR_SQ=BAR**2; AR1_SQ=_A_1**2;
_A_1=ABS(_A_1);

LABEL A_SQ     ='SQUARED BETA 1 WEIGHT WITH PROC REG';
LABEL B_SQ     ='SQUARED BETA 2 WEIGHT WITH PROC REG';
LABEL AAR_SQ   ='CORRECTED SQUARED BETA 1 WEIGHT WITH PROC AUTOREG';
LABEL BAR_SQ   ='CORRECTED SQUARED BETA 2 WEIGHT WITH PROC AUTOREG';
LABEL A        ='BETA WEIGHT 1 WITH PROC REG';
LABEL B        ='BETA WEIGHT 2 WITH PROC REG';
LABEL AAR      ='CORRECTED BETA WEIGHT 1 WITH PROC AUTOREG';
LABEL BAR      ='CORRECTED BETA WEIGHT 2 WITH PROC AUTOREG';
LABEL _A_1     ='AUTOREGRESSION WEIGHT';
LABEL AR1_SQ   ='SQUARED AUTOREGRESSION WEIGHT';
LABEL MSE      ='PROC REG MEAN SQUARE ERROR';
LABEL STERR_A  ='BETA 1 STANDARD ERROR';
LABEL STERR_B  ='BETA 2 STANDARD ERROR';
LABEL AR_MSE   ='PROC AUTOREG MEAN SQUARE ERROR';
LABEL STERRAAR ='CORRECTED BETA 1 STANDARD ERROR';
LABEL STERRBAR ='CORRECTED BETA 2 STANDARD ERROR';

     OUTPUT; *** obtain square values;

PROC DELETE DATA=GENERATEA1;
PROC DELETE DATA=GENERATEB1;
PROC DELETE DATA=GENERATEC1;
PROC DELETE DATA=GENERATEA;
PROC DELETE DATA=GENERATEB;

DATA GENERATE&J; SET GENERATE&J; SCENARIO=&SCENARI; OUTPUT;
     KEEP A   B   STERR_A  STERR_B  MSE  A_SQ   B_SQ
          AAR BAR STERRAAR STERRBAR AR_MSE _A_1 AAR_SQ BAR_SQ AR1_SQ SCENARIO;

   PROC APPEND BASE=AUTOREG.RESULT1 (CNTLLEV=MEMBER);      *** accumulate the results;
   PROC DELETE DATA=GENERATE&J;

  %END;
%MEND AUTOREG;


%AUTOREG (200,500,1.0,.00,.45,.30,.25,01); *** parameters for sixteen conditions;
%AUTOREG (200,500,.95,.05,.45,.30,.25,02);
%AUTOREG (200,500,.90,.10,.45,.30,.25,03);
%AUTOREG (200,500,.85,.15,.45,.30,.25,04);
%AUTOREG (200,500,.80,.20,.45,.30,.25,05);
%AUTOREG (200,500,.75,.25,.45,.30,.25,06);
%AUTOREG (200,500,.70,.30,.45,.30,.25,07);
%AUTOREG (200,500,.65,.35,.45,.30,.25,08);
%AUTOREG (200,500,.60,.40,.45,.30,.25,09);
%AUTOREG (200,500,.55,.45,.45,.30,.25,10);
%AUTOREG (200,500,.50,.50,.45,.30,.25,11);
%AUTOREG (200,500,.45,.55,.45,.30,.25,12);
%AUTOREG (200,500,.40,.60,.45,.30,.25,13);
%AUTOREG (200,500,.35,.65,.45,.30,.25,14);
%AUTOREG (200,500,.30,.70,.45,.30,.25,15);
%AUTOREG (200,500,.25,.75,.45,.30,.25,16);


DATA AUTOREG; SET AUTOREG.RESULT1;         *** accessing the output file;

PROC SUMMARY PRINT VARDEF=N MAXDEC=3 FW=8;   *** summarize results for six conditions;
        CLASS SCENARIO;  *** summarizes results;
             VAR  A   B   STERR_A  STERR_B  MSE         A_SQ   B_SQ
                  AAR BAR STERRAAR STERRBAR AR_MSE _A_1 AAR_SQ BAR_SQ AR1_SQ;
QUIT;
*************************************************************************************;
